#!/usr/bin/env python3
#
# Parses and checks non-phone devices from CUCM.

import json
from cmk.base.plugins.agent_based.agent_based_api.v1 import register, Result, Service, State


# Convert JSON entries into dictionaries indexed by name.
def parse_cucm(string_table):
    lookup = {}

    for row in string_table:
        device = json.loads(row[0])
        name = device["name"]
        lookup[name] = device

    return lookup

register.agent_section(
    name="cucm_chk",
    parse_function=parse_cucm
)


# Produce a list of services based on the parsed devices.
def discover_cucm(section):
    for name, details in sorted(section.items()):
        model_name = details["model_name"]
        ip = details.get("ip")
        gui_name = "%s %s (%s)" % (model_name, name, ip)
        yield Service(item=gui_name, parameters={"name": name})


# Given a specific device, look it up in the parsed devices, and produce
# results on that service based upon the devices' status.
def check_cucm(item, params, section):
    name = params["name"]
    device = section.get(name)

    if device is None:
        yield Result(state=State.WARN, summary="Not appearing in CUCM API")
        return

    status = device.get("status")

    if status is None:
        yield Result(state=State.WARN, summary="No status for this in CUCM API")
    elif status == "Registered":
        yield Result(state=State.OK, summary="Registered")
    elif ["Unregistered", "Rejected", "PartiallyRegistered", "Unknown"].count(status) == 1:
        summary = status
        msg = device.get("status_reason")
        if msg:
            summary += " " + msg
        yield Result(state=State.WARN, summary=summary)
    else:
        yield Result(state=State.WARN, summary="Unknown status: %s" % status)


register.check_plugin(
    name="cucm_chk",
    service_name="CUCM %s",
    discovery_function=discover_cucm,
    check_function=check_cucm,
    check_default_parameters={},
    check_ruleset_name="cucm_chk",
)
